/***************************************************************************
 *   Copyright (C) 2009 by Sergey Yakushev                                 *
 *   yakushevs@list.ru                                                     *
 *                                                                         *
 *   This is a plugin generated from the QGIS plugin template              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

/**
 * \file directioneditormaptool.cpp
 * \brief implemetation direction editor maptool
 */


//qt includes
#include <qevent.h>

// Qgis includes
#include "qgsmapcanvas.h"
#include "qgsvectorlayer.h"
#include "qgsgeometry.h"
#include "qgsmaptopixel.h"

// VrpPlugin includes
#include "settings.h"
#include "directioneditormaptool.h"

//standard includes
#include <cmath>
VrpPluginDirectionEditorMapTool::VrpPluginDirectionEditorMapTool( QgsMapCanvas* theCanvas,
			VrpPluginSettings & theSettings )
			: QgsMapTool( theCanvas ), mSettings( theSettings )
{

}
VrpPluginDirectionEditorMapTool::~VrpPluginDirectionEditorMapTool()
{
	
}
void VrpPluginDirectionEditorMapTool::canvasPressEvent( QMouseEvent* theEvent )
{
	QgsVectorLayer *vl = mSettings.roadsLayer();
	if ( vl == NULL )
		return;
	int directionFieldId = mSettings.directionFieldId();
	if ( directionFieldId == -1 )
		return;

	QgsPoint pt = mCanvas->getCoordinateTransform()->toMapCoordinates( theEvent->x(), theEvent->y() );
        double r = std::abs( mCanvas->mapUnitsPerPixel() )*15;
        double minimum = 1e10;
	QgsFeature minFeature;
	
	QgsAttributeList fetchAttr;
	fetchAttr.push_back( directionFieldId );
	vl->select( fetchAttr , mCanvas->extent() );
	QgsFeature feature;
	while ( vl->nextFeature( feature ) )
	{
		QgsPolyline pl = feature.geometry()->asPolyline();
		if (pl.size() == 0)
			continue;
		QgsAttributeMap attr = feature.attributeMap();
	
		
		QgsPoint pt1;
		QgsPoint pt2;
		
		bool isFirstPoint = true;
		double x=0,y=0;
		
		VrpPluginSettings::DirectionType directionType = mSettings.directionType(feature);
		QgsPolyline::iterator it2;
		for (it2 = pl.begin(); it2 != pl.end(); ++it2)
		{
			pt2 = QgsPoint(it2->x(),it2->y());
			
			if ( !isFirstPoint )
			{
				double d = sqrt(pow((pt1.x()+pt2.x())/2-pt.x(),2) + pow((pt1.y()+pt2.y())/2 - pt.y(), 2));
				if (d < minimum)
				{
					minimum = d;
					minFeature = feature;
				}
				
			}
			
			isFirstPoint = false;
			
			pt1 = pt2;		
		}
	}
	if ( minimum < r )
	{
		VrpPluginSettings::DirectionType directionType = mSettings.directionType(minFeature);
		if ( directionType == VrpPluginSettings::Both )
			directionType = VrpPluginSettings::FirstPointToLastPoint;
		else if ( directionType == VrpPluginSettings::FirstPointToLastPoint )
			directionType = VrpPluginSettings::LastPointToFirstPoint;
		else
			directionType = VrpPluginSettings::Both;
		
		vl->changeAttributeValue( minFeature.id(), mSettings.directionFieldId(), directionType );
		
		mCanvas->refresh();
	}


}
bool VrpPluginDirectionEditorMapTool::isEditTool( )
{
	return true;
}
