/***************************************************************************
  plugin.cpp
  This plugin solve the vechicle routing problem and etc.
  -------------------
         begin                : 2009-04-20
         copyright            : (C) Sergey Yakushev 2009
         email                : yakushevs@list.ru

 ***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/*  $Id: plugin.cpp 9327 2008-09-14 11:18:44Z jef $ */


// QGIS Specific includes
#include <qgisinterface.h>
#include <qgisgui.h>
#include <qgsmapcanvas.h>
#include <qgsmaplayerregistry.h>
#include <qgsvectorlayer.h>
#include <qgsvectordataprovider.h>
#include <qgsgeometry.h>
#include <qgsproject.h>
#include <qgsmaptoolemitpoint.h>
#include <qgsdistancearea.h>
#include <qgsmaprenderer.h>

#include "qgsapplication.h"

// VRP plugin includes
#include "plugin.h"
#include "plugingui.h"
#include "utils.h"
#include "settings.h"
#include "directioneditormaptool.h"
#include "shortestpathwidget.h"

//
// Qt4 Related Includes
//

#include <QAction>
#include <QToolBar>
#include <QPainter>
#include <QDockWidget>

// standart includes
#include <iostream>
#include <cmath>

static const char * const sIdent = "$Id: plugin.cpp 9327 2009-04-20 10:09:44Z YEKST $";
static const QString sName = QObject::tr( "VRP" );
static const QString sDescription = QObject::tr( "It solve vehicle routing poblem and etc." );
static const QString sPluginVersion = QObject::tr( "Version 0.1" );
static const QgisPlugin::PLUGINTYPE sPluginType = QgisPlugin::UI;

//////////////////////////////////////////////////////////////////////
//
// THE FOLLOWING METHODS ARE MANDATORY FOR ALL PLUGINS
//
//////////////////////////////////////////////////////////////////////

/**
 * Constructor for the plugin. The plugin is passed a pointer
 * an interface object that provides access to exposed functions in QGIS.
 * @param theQGisInterface - Pointer to the QGIS interface object
 */
VrpPlugin::VrpPlugin( QgisInterface * theQgisInterface ):
    QgisPlugin( sName, sDescription, sPluginVersion, sPluginType ),
    mQGisIface( theQgisInterface )
{
	mSettings = VrpPluginSettings::defaultSettings();
	mDirectionEditorMapTool = NULL;
	mQShortestPathDock = NULL;
//	myMapTool = NULL;
}

VrpPlugin::~VrpPlugin()
{

}

/*
 * Initialize the GUI interface for the plugin - this is only called once when the plugin is
 * added to the plugin registry in the QGIS application.
 */
void VrpPlugin::initGui()
{
	// create shorttest path dock
	mQShortestPathDock = new VrpPluginShortestPathWidget( mQGisIface->mainWindow() , this );
	mQGisIface->addDockWidget( Qt::LeftDockWidgetArea, mQShortestPathDock );

	// Create the action for tool
	mQSettingsAction	= new QAction( QIcon( ":/vrp/vrp.png" ), tr( "VRP settings" ), this );
	mQShowDirectionAction	= new QAction( QIcon( ":/vrp/showdirect.png" ), tr( "Show roads direction" ), this);
	mQEditDirectionAction	= new QAction( QIcon( ":/vrp/editdirect.png" ), tr("Edit roads direction"), this );
	
	// Set the what's this text
	mQSettingsAction->setWhatsThis( tr("Vehicle routing problem solvers settings") );
	mQShowDirectionAction->setWhatsThis( tr("Roads direction viewer") );
	mQEditDirectionAction->setWhatsThis( tr("Roads direction editor") );

	mQShowDirectionAction->setCheckable( true );
	mQEditDirectionAction->setCheckable( true );	

	setGuiElementsToDefault();

	mDirectionEditorMapTool = new VrpPluginDirectionEditorMapTool( mQGisIface->mapCanvas(), mSettings );
	mDirectionEditorMapTool->setAction( mQEditDirectionAction );

//	Connect the action to slots
 	connect( mQSettingsAction, SIGNAL( triggered() ), this, SLOT( property() ) );
	connect( mQShowDirectionAction, SIGNAL( triggered() ), this, SLOT( onShowDirection() ) );
  	connect( mQEditDirectionAction, SIGNAL( triggered( bool ) ), this, SLOT( onEditDirection( bool ) ) );


// 	Add the icons to the toolbar
	mQGisIface->addToolBarIcon( mQShowDirectionAction );
	mQGisIface->addToolBarIcon( mQEditDirectionAction );

	mQGisIface->addPluginToMenu( tr( "VRP" ), mQSettingsAction );
	mQGisIface->addPluginToMenu( tr( "VRP" ), mQShowDirectionAction );
	mQGisIface->addPluginToMenu( tr( "VRP" ), mQEditDirectionAction );

	connect( mQGisIface->actionToggleEditing(), SIGNAL( triggered() ), this, SLOT( onToggleEditing() ) );
	connect( mQGisIface->mapCanvas(), SIGNAL( renderComplete(QPainter*) ), this, SLOT( render(QPainter*) ) );
	connect( mQGisIface->mainWindow(), SIGNAL( projectRead() ), this, SLOT( projectRead() ) );
	// FIXME:
	connect( mQGisIface->mainWindow(), SIGNAL( newProject(bool)), this, SLOT(newProject()) );
}

// Unload the plugin by cleaning up the GUI
void VrpPlugin::unload()
{
	// remove the GUI
	mQGisIface->removePluginMenu( tr("VRP"), mQSettingsAction );
	mQGisIface->removePluginMenu( tr("VRP"), mQShowDirectionAction );
  	mQGisIface->removePluginMenu( tr("VRP"), mQEditDirectionAction );

	mQGisIface->removeToolBarIcon( mQShowDirectionAction );
  	mQGisIface->removeToolBarIcon( mQEditDirectionAction );

 	// disconnect
  	disconnect( mQGisIface->mapCanvas(), SIGNAL( renderComplete(QPainter*) ), this, SLOT( render(QPainter*) ) );
	disconnect( mQGisIface->mainWindow(), SIGNAL( projectRead() ), this, SLOT( projectRead() ) );
	
	// unset maptool
	mQGisIface->mapCanvas()->unsetMapTool( mDirectionEditorMapTool );

	delete mDirectionEditorMapTool;
  	delete mQSettingsAction;
  	delete mQEditDirectionAction;
	delete mQShowDirectionAction;
	delete mQShortestPathDock;
}
void VrpPlugin::setGuiElementsToDefault()
{
	if ( mSettings.roadsLayer()==NULL )
	{
		mQShowDirectionAction->setEnabled( false );
	}else
	{
		mQShowDirectionAction->setEnabled( true );
	}
	mQShowDirectionAction->setChecked( false );
	mQEditDirectionAction->setChecked( false );
	mQEditDirectionAction->setEnabled( false );
		

}
void VrpPlugin::onToggleEditing()
{
	QgsVectorLayer *vl = mSettings.roadsLayer();
	if (vl == NULL)
		return;
	if ( vl->isEditable() && mQShowDirectionAction->isChecked() )
	{
		mQEditDirectionAction->setEnabled ( true );
	}else
	{
		mQEditDirectionAction->setEnabled ( false );	
	}
		
}
//method defined in interface
void VrpPlugin::help()
{
  //implement me!
}

void VrpPlugin::onShowDirection()
{
	QgsVectorLayer *vl = mSettings.roadsLayer();
	if (vl == NULL)
		return;

	mQGisIface->mapCanvas()->refresh();
	if ( !mQShowDirectionAction->isChecked() )
	{
		mQEditDirectionAction->setEnabled( false );
		mQGisIface->mapCanvas()->unsetMapTool( mDirectionEditorMapTool );
	}else if ( vl->isEditable() )
	{
		mQEditDirectionAction->setEnabled( true );
	}

}
void VrpPlugin::onEditDirection( bool b)
{	
	if ( b)
		mQGisIface->mapCanvas()->setMapTool( mDirectionEditorMapTool );
	else
		mQGisIface->mapCanvas()->unsetMapTool( mDirectionEditorMapTool );
}

void VrpPlugin::property()
{
	VrpPluginGui dlg(mSettings, mQGisIface->mainWindow(), QgisGui::ModalDialogFlags );
	if ( !dlg.exec() )
		return;
	mSettings = dlg.settings();
	
	QgsProject::instance()->writeEntry("VRPplugin", "/defaultDirection", mSettings.mDefaultDirection);
	QgsProject::instance()->writeEntry("VRPplugin", "/directionField", mSettings.mDirection);
	QgsProject::instance()->writeEntry("VRPplugin", "/speedField", mSettings.mSpeed);
	QgsProject::instance()->writeEntry("VRPplugin", "/defaultSpeed", mSettings.mDefaultSpeed);
	QgsProject::instance()->writeEntry("VRPplugin", "/layer", mSettings.mLayer);
	QgsProject::instance()->writeEntry("VRPplugin", "/offroadSpeed", mSettings.mOffroadSpeed);
	QgsProject::instance()->writeEntry("VRPplugin", "/speedUnit", mSettings.speedUnit().name() );
	setGuiElementsToDefault();
}
void VrpPlugin::newProject()
{
	mSettings = VrpPluginSettings::defaultSettings();
	setGuiElementsToDefault();
}
void VrpPlugin::projectRead()
{
	VrpPluginSettings s;
	
	int dd 			= QgsProject::instance()->readNumEntry("VRPplugin", "/defaultDirection");
	s.mDirection		= QgsProject::instance()->readEntry("VRPplugin", "/directionField");
	s.mSpeed		= QgsProject::instance()->readEntry("VRPplugin", "/speedField");
	s.mDefaultSpeed		= QgsProject::instance()->readDoubleEntry("VRPplugin", "/defaultSpeed");
	s.mLayer		= QgsProject::instance()->readEntry("VRPplugin", "/layer");
	s.mOffroadSpeed		= QgsProject::instance()->readDoubleEntry("VRPplugin", "/offroadSpeed");
	s.setSpeedUnit( QgsProject::instance()->readEntry("VRPplugin","/speedUnit") );

	if (dd == 1)
		s.mDefaultDirection = VrpPluginSettings::FirstPointToLastPoint;
	else if (dd == 2)
		s.mDefaultDirection = VrpPluginSettings::LastPointToFirstPoint;
	else if (dd == 3)
		s.mDefaultDirection = VrpPluginSettings::Both;

	if ( s.testSettings() )
	{
		mSettings = s;
	}else
	{
		mSettings = VrpPluginSettings::defaultSettings();
	}
	setGuiElementsToDefault();
}

QgisInterface* VrpPlugin::iface()
{
	return mQGisIface;
}
VrpPluginSettings* VrpPlugin::settings()
{
	return &mSettings;
}
void VrpPlugin::render(QPainter *painter)
{
	QgsVectorLayer *vl = mSettings.roadsLayer();
	if ( vl == NULL )
		return;
	if ( mQShowDirectionAction->isChecked() )
	{
		QgsCoordinateTransform ct;
		ct.setDestCRS( mQGisIface->mapCanvas()->mapRenderer()->destinationSrs() );
		ct.setSourceCrs( vl->crs() );	

	        QgsAttributeList fetchAttr;
	        if (mSettings.directionFieldId() != -1)
	                fetchAttr.push_back( mSettings.directionFieldId() );

		vl->select( fetchAttr );
		QgsFeature feature;
		while ( vl->nextFeature(feature) )
		{
			QgsAttributeMap attr = feature.attributeMap();
			QgsPolyline pl = feature.geometry()->asPolyline();
			if (pl.size() == 0)
				continue;
			QgsPoint pt1;
			QgsPoint pt2;
			bool isFirstPoint = true;
			double x=0,y=0;
			VrpPluginSettings::DirectionType directionType = mSettings.directionType(feature);
			QgsPolyline::iterator it2;
			for (it2 = pl.begin(); it2 != pl.end(); ++it2)
			{
				pt2 = mQGisIface->mapCanvas()->getCoordinateTransform()
					->transform( ct.transform( QgsPoint(it2->x(),it2->y())) );
				
				if ( !isFirstPoint && sqrt(pow(pt1.x()-pt2.x(),2)+pow(pt1.y()-pt2.y(),2)) > mArrowSize*3)
					renderArrow( painter, pt1.x(), pt1.y(), pt2.x(), pt2.y(), directionType );
				isFirstPoint = false;
				pt1 = pt2;
			}
		}	
	}
}

void VrpPlugin::renderArrow( QPainter *painter, double x1, double y1, double x2, double y2, 
	VrpPluginSettings::DirectionType directionType)
{
	double length = sqrt( pow( x2-x1, 2.0) + pow( y2-y1, 2.0 ) );
	double Cos = (x2-x1)/length;
	double Sin = (y2-y1)/length;
	double centerX = (x1+x2)/2;
	double centerY = (y1+y2)/2;
	double r = mArrowSize;
	
	QPointF pt1(centerX - Sin*r, centerY + Cos*r);
	QPointF pt2(centerX + Sin*r, centerY - Cos*r);
	
	QVector<QPointF> tmp;
	tmp.resize(3);
	tmp[1] = pt1;
	tmp[2] = pt2;

	
	if ( directionType == VrpPluginSettings::Both || directionType == VrpPluginSettings::FirstPointToLastPoint )
	{
		tmp[0] = QPointF( centerX + Cos*r*2, centerY + Sin*r*2 );
		painter->drawPolygon(tmp);
	}
	if ( directionType == VrpPluginSettings::Both || directionType == VrpPluginSettings::LastPointToFirstPoint )
	{
		tmp[0] = QPointF( centerX - Cos*r*2, centerY - Sin*r*2 );
		painter->drawPolygon(tmp);
	}
}

AdjacencyMatrix VrpPlugin::adjacencyMatrix( const QList< QgsPoint >& additionPoints)
{
	double multipler = 1.0; // convert distance to metric system
	if ( mQGisIface->mapCanvas()->mapUnits() == QGis::Feet )
		multipler = 0.3048; // feet in meter

	AdjacencyMatrix m;
	AdjacencyMatrix addon2;
	AdjacencyMatrixAddon addon;	

	QgsVectorLayer *vl = mSettings.roadsLayer();
	if (vl == NULL)
		return m;
	// translate vector layers coordinate to the project coorditate
        QgsCoordinateTransform ct;
        ct.setDestCRS( mQGisIface->mapCanvas()->mapRenderer()->destinationSrs() );
        ct.setSourceCrs( vl->crs() );

	QgsDistanceArea da;
	da.setSourceCrs( mQGisIface->mapCanvas()->mapRenderer()->destinationSrs().srsid() );
	da.setProjectionsEnabled( true );

	QgsAttributeList attrList;
	if (mSettings.directionFieldId() != -1)
		attrList.push_back( mSettings.directionFieldId() );
	if ( mSettings.speedFieldId() != -1 )
		attrList.push_back( mSettings.speedFieldId() );

	vl->select( attrList );
	QgsFeature feature;
	while ( vl->nextFeature( feature ) )
	{
		VrpPluginSettings::DirectionType directionType = mSettings.directionType( feature );
		double speed = mSettings.speed( feature );

		QgsPoint pt1;
		QgsPoint pt2;
		bool firstPoint = true;

		QgsPolyline pl = feature.geometry()->asPolyline();
		QgsPolyline::iterator it;
		for (it = pl.begin(); it != pl.end(); ++it)
		{
			pt2 = ct.transform(*it);
			if ( !firstPoint )
			{
				QList< QgsPoint> pl2;
				pl2.push_back(pt1);
				pl2.push_back(pt2);
				double segmentLength = da.measureLine(pl2)*multipler;
				//std::cerr << segmentLength << "/" << speed << "=" << segmentLength/speed << "\n";
				if ( directionType == VrpPluginSettings::FirstPointToLastPoint || 
					directionType == VrpPluginSettings::Both )
				{
					m[ pt1 ][ pt2 ] = ArcAttributes(segmentLength, segmentLength/speed, feature.id() );
				}
				if ( directionType == VrpPluginSettings::LastPointToFirstPoint ||
					directionType == VrpPluginSettings::Both )
				{
					m[ pt2 ][ pt1 ] = ArcAttributes(segmentLength, segmentLength/speed, feature.id() );
				}
			}
			firstPoint = false;
			pt1 = pt2;
		}
	}

	AdjacencyMatrix::iterator it1;
	AdjacencyMatrixString::iterator it2;

	QList< QgsPoint >::const_iterator points_it;
	for( points_it = additionPoints.constBegin(); points_it != additionPoints.constEnd(); ++points_it)
	{
		double minD = infinity();
		for ( it1 = m.begin(); it1 != m.end(); ++it1 )
		{
			for ( it2 = it1->second.begin(); it2 != it1->second.end(); ++it2)
			{
			QgsPoint center;
				double d = distance(
					it1->first, 
					it2->first,
					*points_it,
					center
					);
				if (d < minD)
				{
					minD = d;
				}
			}
		}
		if (minD >= infinity())
			continue;

		for ( it1 = m.begin(); it1 != m.end(); ++it1 )
		{
			for ( it2 = it1->second.begin(); it2 != it1->second.end(); ++it2)
			{
				QgsPoint center;
				double d = distance(
					it1->first, 
					it2->first,
					*points_it,
					center
					);
				if (std::abs(d- minD) < 0.000001)
				{
					addon[it1->first][it2->first].push_back(center);
					QList< QgsPoint > l;
					
					l.push_back( center );
					l.push_back( *points_it );
	
					double segmentLength = da.measureLine( l ) / 1000;

					addon2[ center ][ *points_it ] = ArcAttributes( segmentLength, 
									segmentLength / mSettings.mOffroadSpeed, -1 );
					addon2[ *points_it ][ center ] = ArcAttributes( segmentLength, 
									segmentLength / mSettings.mOffroadSpeed, -1 );
				//
				}
			}
		}
	}
	for (it1 = addon2.begin(); it1 != addon2.end(); ++it1)
	{
		for (it2 = it1->second.begin(); it2 != it1->second.end(); ++it2)
		{
			m[ it1->first ][ it2->first ] = it2->second;
		}
	}
	
	AdjacencyMatrixStringAddon::iterator	addon_it2;
	AdjacencyMatrixAddon::iterator		addon_it1;
	for (addon_it1 = addon.begin(); addon_it1 != addon.end(); ++addon_it1)
	{
		for (addon_it2 = addon_it1->second.begin(); addon_it2 != addon_it1->second.end(); ++addon_it2)
		{
			addon_it2->second.sort( AddonSort(addon_it1->first, addon_it2->first) );
			addon_it2->second.push_back( addon_it2->first );

			QgsPoint pt2 = addon_it1->first;
			QgsPoint pt1;
			std::list< QgsPoint >::const_iterator it;
			for (it = addon_it2->second.begin(); it != addon_it2->second.end(); ++it)
			{
				pt1 = pt2;
				pt2 = *it;
				QList< QgsPoint > l;
				l.push_back( pt1 );
				l.push_back( pt2 );

				double segmentLength = da.measureLine( l )*multipler;
				double lineLength = m[ addon_it1->first ][ addon_it2->first ].mCost;
				double lineTime   = m[ addon_it1->first ][ addon_it2->first ].mTime;
				int featureId     = m[ addon_it1->first ][ addon_it2->first ].mFeatureId;
				
				m[ pt1 ][ pt2 ] = ArcAttributes( segmentLength, lineTime*segmentLength/lineLength, 
								featureId );
			}
		}
	}
	return m;
}
// return distance between line of two points and center
double VrpPlugin::distance(const QgsPoint& p1, const QgsPoint& p2, const QgsPoint& p, QgsPoint& center)
{
	// first line
	double A1,B1,C1;
	A1 = p1.y() - p2.y();
	B1 = p2.x() - p1.x();
	C1 = p1.x()*(-A1) + p1.y()*(-B1);
				
	// second line. First and second line is perpendicular.
	double A2,B2,C2;
	A2 = B1;
	B2 = -A1;
	C2 = -p.x()*A2 - p.y()*B2;

	// union point
	double x,y,det;
	det = A1*B2 - B1*A2;
	x = (C2*B1 - B2*C1)/det;
	y = (-A1*C2 + C1*A2)/det;
	
	center = QgsPoint(x,y);
	
	det = sqrt(A1*A1+B1*B1);
	A1 /= det;
	B1 /= det;
	C1 /= det;
	if ( std::min(p1.x(),p2.x()) <= x && std::max(p1.x(),p2.x()) >= x &&
		std::min(p1.y(),p2.y()) <= y && std::max(p1.y(),p2.y()) >= y )
		return std::abs( A1*p.x()+B1*p.y()+C1 );
	return infinity();

}
//////////////////////////////////////////////////////////////////////////
//
//
//  THE FOLLOWING CODE IS AUTOGENERATED BY THE PLUGIN BUILDER SCRIPT
//    YOU WOULD NORMALLY NOT NEED TO MODIFY THIS, AND YOUR PLUGIN
//      MAY NOT WORK PROPERLY IF YOU MODIFY THIS INCORRECTLY
//
//
//////////////////////////////////////////////////////////////////////////


/**
 * Required extern functions needed  for every plugin
 * These functions can be called prior to creating an instance
 * of the plugin class
 */
// Class factory to return a new instance of the plugin class
QGISEXTERN QgisPlugin * classFactory( QgisInterface * theQgisInterfacePointer )
{
  return new VrpPlugin( theQgisInterfacePointer );

}
// Return the name of the plugin - note that we do not user class members as
// the class may not yet be insantiated when this method is called.
QGISEXTERN QString name()
{
  return sName;
}

// Return the description
QGISEXTERN QString description()
{
  return sDescription;
}

// Return the type (either UI or MapLayer plugin)
QGISEXTERN int type()
{
  return sPluginType;
}

// Return the version number for the plugin
QGISEXTERN QString version()
{
  return sPluginVersion;
}

// Delete ourself
QGISEXTERN void unload( QgisPlugin * thePluginPointer )
{
  delete thePluginPointer;
}
