/***************************************************************************
 *   Copyright (C) 2009 by Sergey Yakushev                                 *
 *   yakushevs@list.ru                                                     *
 *                                                                         *
 *   This is a plugin generated from the QGIS plugin template              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "plugingui.h"
#include "qgscontexthelp.h"

//qt includes
#include <qlabel.h>
#include <qcombobox.h>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <qdialogbuttonbox.h>
#include <qtabwidget.h>
#include <qspinbox.h>
#include <qmessagebox.h>

// Qgis includes
#include "qgsfield.h"
#include "qgsmaplayerregistry.h"
#include "qgsvectordataprovider.h"
#include "qgsvectorlayer.h"

//standard includes


VrpPluginGui::VrpPluginGui(const VrpPluginSettings& s, QWidget* parent, Qt::WFlags fl )
    : QDialog( parent, fl )
{
	// create base widgets;
	setWindowTitle("VRP plugin");
	QTabWidget *tab = new QTabWidget(this);
	QVBoxLayout *v= new QVBoxLayout(this);
	v->addWidget(tab);
	QDialogButtonBox *bb = new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel, Qt::Horizontal, this);
	connect(bb, SIGNAL(accepted()), this, SLOT(on_buttonBox_accepted()) );
	connect(bb, SIGNAL(rejected()), this, SLOT(on_buttonBox_rejected()) );
	v->addWidget(bb);

	// transportation layer
	QFrame *frame = new QFrame();
	tab->addTab(frame, tr("Transportation layer"));
	v = new QVBoxLayout(frame);
	QLabel *l = new QLabel(tr("Layer"), frame);
	mcbLayers = new QComboBox(frame);
	QHBoxLayout *h = new QHBoxLayout();

	h->addWidget(l);
	h->addWidget(mcbLayers);
	v->addLayout(h);

	h = new QHBoxLayout();
	l = new QLabel(tr("Direction field"), frame);
	mcbDirection = new QComboBox(frame);
	h->addWidget(l);
	h->addWidget(mcbDirection);
	v->addLayout(h);

	h = new QHBoxLayout();
	l = new QLabel(tr("Speed field"), frame);
	mcbSpeed = new QComboBox(frame);
	h->addWidget(l);
	h->addWidget(mcbSpeed);
	v->addLayout(h);
	
	frame = new QFrame(tab);
	tab->addTab( frame, tr("Default settings"));
	v = new QVBoxLayout(frame);
	h = new QHBoxLayout();
	l = new QLabel( tr("Direction"), frame);
	mcbDirectionDefault = new QComboBox(frame);
	mcbDirectionDefault->insertItem(0,tr("Both direction") );
	mcbDirectionDefault->insertItem(1,tr("first points, last point") );
	mcbDirectionDefault->insertItem(2,tr("last points, first point") );
	connect( mcbLayers, SIGNAL(currentIndexChanged(int)), this, SLOT(on_mcbLayers_selectItem()) );

	h->addWidget(l);
	h->addWidget(mcbDirectionDefault);
	v->addLayout(h);
	
	h = new QHBoxLayout(frame);
	l = new QLabel(tr("Cost"), frame);
	h->addWidget(l);
	l = new QLabel(tr("lines length"), frame);
	h->addWidget(l);
	v->addLayout(h);

	h = new QHBoxLayout(frame);
	l = new QLabel(tr("Speed"), frame);
	msbSpeedDefault = new QSpinBox(frame);
	msbSpeedDefault->setMinimum(1);
	msbSpeedDefault->setMaximum(10000000);
	h->addWidget(l);
	h->addWidget(msbSpeedDefault);
	v->addLayout(h);
	
	h = new QHBoxLayout();
	l = new QLabel(tr("Off road speed"), frame);
	msbOffroadSpeed = new QSpinBox(frame);
	msbOffroadSpeed->setMinimum(1);
	msbOffroadSpeed->setMaximum(10000000);

	h->addWidget(l);
	h->addWidget(msbOffroadSpeed);
	v->addLayout(h);

	frame = new QFrame(tab);
	tab->addTab( frame, tr("Units") );
	v = new QVBoxLayout( frame );
	h = new QHBoxLayout();
	l = new QLabel(tr("Unit of speed"));
	mcbUnitOfSpeed = new QComboBox(this);
	h->addWidget(l);
	h->addWidget( mcbUnitOfSpeed );	
	v->addLayout( h );

	mcbUnitOfSpeed->insertItem( 0, tr("km/h") );
	mcbUnitOfSpeed->insertItem( 0, tr("m/s") );
	if ( s.speedUnit().name() == "km/h" ) 
		mcbUnitOfSpeed->setCurrentIndex( 1 );
	else 
		mcbUnitOfSpeed->setCurrentIndex( 0 );
	// fill list of layers
	QMap<QString, QgsMapLayer*> mapLayers = QgsMapLayerRegistry::instance()->mapLayers();
    	QMap<QString, QgsMapLayer*>::iterator layer_it = mapLayers.begin();

      	for ( ; layer_it != mapLayers.end(); ++layer_it )
        {
		QgsVectorLayer* vl = dynamic_cast<QgsVectorLayer*>( layer_it.value() );
	        if ( !vl )
			continue;
		if ( vl->geometryType() != QGis::Line )
			continue;
		mcbLayers->insertItem( 0, vl->name() );	
	}
	
	//sets current settings
	msbSpeedDefault->setValue( static_cast<int>( s.mDefaultSpeed ) );	
	msbOffroadSpeed->setValue( static_cast<int>( s.mOffroadSpeed ) );

	int idx = mcbLayers->findText( s.mLayer );
	if (idx != -1)
	{
		mcbLayers->setCurrentIndex( idx );
	}
		
	idx = mcbDirection->findText( s.mDirection );
	if (idx != -1)
		mcbDirection->setCurrentIndex( idx );
	
	idx = mcbSpeed->findText( s.mSpeed );
	if (idx != -1)
		mcbSpeed->setCurrentIndex( idx );
	
	
	switch( s.mDefaultDirection )
	{
	case VrpPluginSettings::Both:
		mcbDirectionDefault->setCurrentIndex(0);
		break;
	case VrpPluginSettings::FirstPointToLastPoint:
		mcbDirectionDefault->setCurrentIndex(1);
		break;
	case VrpPluginSettings::LastPointToFirstPoint:
		mcbDirectionDefault->setCurrentIndex(2);
		break;
	}
}

VrpPluginGui::~VrpPluginGui()
{
}
QgsVectorLayer* VrpPluginGui::selectedLayer()
{
	QMap<QString, QgsMapLayer*> mapLayers = QgsMapLayerRegistry::instance()->mapLayers();
    	QMap<QString, QgsMapLayer*>::iterator layer_it = mapLayers.begin();

      	for ( ; layer_it != mapLayers.end(); ++layer_it )
        {
		QgsVectorLayer* vl = dynamic_cast<QgsVectorLayer*>( layer_it.value() );
	        if ( !vl )
			continue;
		if ( vl->geometryType() != QGis::Line )
			continue;
		if ( vl->name() == mcbLayers->currentText() )
			return vl;
	}

	return NULL;
}
void VrpPluginGui::on_mcbLayers_selectItem()
{
	mcbDirection->clear();
	mcbSpeed->clear();

	mcbDirection->insertItem( 0, tr("Always use default") );
	mcbSpeed->insertItem(0, tr("Always use default"));

	QgsVectorLayer* vl = selectedLayer();
	if ( !vl )
		return;

	QgsVectorDataProvider* provider = vl->dataProvider();
	if ( !provider )
		return;
	
	const QgsFieldMap& fields = provider->fields();
	QgsFieldMap::const_iterator it;
	for (it = fields.constBegin(); it != fields.constEnd(); ++it)
	{
		QgsField currentField = it.value();
		QVariant currentType = currentField.type();
		if ( currentType == QVariant::Int)
		{
			mcbDirection->insertItem(1, currentField.name());
		}
		if ( currentType == QVariant::Int || currentType == QVariant::Double )
		{
			mcbSpeed->insertItem(1, currentField.name());
		}
	}

}
VrpPluginSettings VrpPluginGui::settings()
{
	VrpPluginSettings s;
	s.mLayer = mcbLayers->currentText();
	if (mcbDirection->currentIndex() > 0)
	{
		s.mDirection = mcbDirection->currentText();
	}else
	{
		s.mDirection = "";
	}
	
	if (mcbSpeed->currentIndex() > 0)
	{
		s.mSpeed = mcbSpeed->currentText();
	}else
	{
		s.mSpeed = "";
	}
	s.mDefaultSpeed = msbSpeedDefault->value();
	s.mOffroadSpeed = msbOffroadSpeed->value();

	switch(mcbDirectionDefault->currentIndex())
	{
	case 0:
		s.mDefaultDirection = VrpPluginSettings::Both;
		break;
	case 1:
		s.mDefaultDirection = VrpPluginSettings::FirstPointToLastPoint;
		break;
	case 2:
		s.mDefaultDirection = VrpPluginSettings::LastPointToFirstPoint;
		break;
	}
	switch( mcbUnitOfSpeed->currentIndex() )
	{
	case 1:
		s.setSpeedUnit("km/h");
		break;
	case 0:
		s.setSpeedUnit("m/s");
		break;
	}
	return s;
}
void VrpPluginGui::on_buttonBox_accepted()
{
	//close the dialog
	if (settings().testSettings())
		accept();
	else
	{
		QMessageBox::critical( this, tr("Settings isn't correct"), tr("The settings isn't correct")); 
	}
}

void VrpPluginGui::on_buttonBox_rejected()
{
	reject();
}

void VrpPluginGui::on_buttonBox_helpRequested()
{
  QgsContextHelp::run( context_id );
}

