/***************************************************************************
 *   Copyright (C) 2009 by Sergey Yakushev                                 *
 *   yakushevs@list.ru                                                     *
 *                                                                         *
 *   This is a plugin generated from the QGIS plugin template              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

/**
 * \file utils.cpp
 * \brief implementation of VRP plugins utils
 */

#include "settings.h"
#include "units.h"

//qt includes

// Qgis includes
#include "qgsvectorlayer.h"
#include "qgsvectordataprovider.h"
#include "qgsmaplayerregistry.h"
//#include "qgspoint.h"
#include "qgsgeometry.h"

//standard includes
VrpPluginSettings::VrpPluginSettings()
{	
	mLayer = "";
	mDirection = "";
	mDirectionFieldId = -2;
	mSpeedFieldId     = -2;
	mDefaultDirection = Both;
	mSpeed = "";
	mDefaultSpeed = 40;
	mOffroadSpeed = 5;
	mSpeedUnit = SpeedUnit( Unit("km", 1000), Unit("h", 3600) );
}

VrpPluginSettings::~VrpPluginSettings()
{
}
VrpPluginSettings VrpPluginSettings::defaultSettings()
{
	VrpPluginSettings s;
	s.mLayer = "";
	s.mDirection = "";
	s.mDefaultDirection = Both;
	s.mSpeed = "";
	s.mDefaultSpeed = 40.0;
	s.mOffroadSpeed = 5;
	s.setSpeedUnit("km/h");
	return s;
}
SpeedUnit VrpPluginSettings::speedUnit() const
{
	return mSpeedUnit;
}
void VrpPluginSettings::setSpeedUnit( QString name )
{
	if (name == "m/s")
		mSpeedUnit = SpeedUnit( Unit("m", 1), Unit("s", 1) );
	else
		mSpeedUnit =  SpeedUnit( Unit("km", 1000), Unit("h", 3600) );
}
int VrpPluginSettings::directionFieldId()
{
	if (mDirectionFieldId > -2)
		return mDirectionFieldId;
	if (mDirection == "")
		return -1;
	QgsVectorLayer *vl = roadsLayer();
	if ( !vl )
		return -1;
	QgsVectorDataProvider *provider = dynamic_cast<QgsVectorDataProvider*>( vl->dataProvider() );
	if ( !provider )
		return -1;
	mDirectionFieldId = provider->fieldNameIndex( mDirection );

	return mDirectionFieldId;
}

VrpPluginSettings::DirectionType VrpPluginSettings::directionType( QgsFeature& feature )
{
	int fieldId = directionFieldId();
	if ( fieldId == -1)
	return mDefaultDirection;
	
	QgsAttributeMap attr = feature.attributeMap();

	QgsAttributeMap::const_iterator it;
	for (it = attr.constBegin(); it != attr.constEnd(); ++it)
	{
		if (it.key() != fieldId)
			continue;
		int direction = it.value().toInt();
		if (direction == Both)
			return Both;
		if (direction == FirstPointToLastPoint)
			return FirstPointToLastPoint;
		if (direction == LastPointToFirstPoint)
			return LastPointToFirstPoint;
		break;
	}

	return mDefaultDirection;
}

int VrpPluginSettings::speedFieldId()
{
	if (mSpeedFieldId > -2)
		return mSpeedFieldId;
	if (mSpeed == "")
		return -1;
	QgsVectorLayer *vl = roadsLayer();
	if ( !vl )
		return -1;
	QgsVectorDataProvider *provider = dynamic_cast<QgsVectorDataProvider*>( vl->dataProvider() );
	if ( !provider )
		return -1;
	mSpeedFieldId = provider->fieldNameIndex( mSpeed );

	return mSpeedFieldId;
}

double VrpPluginSettings::speed( QgsFeature& feature )
{
	int fieldId = speedFieldId();
		
	if ( fieldId == -1)
		return mDefaultSpeed * mSpeedUnit.multipler();
	
	QgsAttributeMap attr = feature.attributeMap();

	QgsAttributeMap::const_iterator it;
	for (it = attr.constBegin(); it != attr.constEnd(); ++it)
	{
		if (it.key() != fieldId)
			continue;
		if ( it.value().toDouble() == 0.0 )
			return mDefaultSpeed * mSpeedUnit.multipler();
		return it.value().toDouble() * mSpeedUnit.multipler();
		break;
	}

	return mDefaultSpeed * mSpeedUnit.multipler();
}

double VrpPluginSettings::offroadSpeed()
{ 
	return mOffroadSpeed;
}

QgsVectorLayer* VrpPluginSettings::roadsLayer()
{
	if (mLayer == "")
		return NULL;
	
	QMap< QString, QgsMapLayer*> m = QgsMapLayerRegistry::instance()->mapLayers();
	QMap< QString, QgsMapLayer*>::const_iterator it;
	for (it = m.constBegin(); it != m.constEnd(); ++it)
	{
		if (it.value()->name() == mLayer)
			return dynamic_cast<QgsVectorLayer*>(it.value());
	}
	return NULL;
}

bool VrpPluginSettings::testSettings()
{
	// implement me
	
	// check default speed
	if (mDefaultSpeed <= 0.0)
		return false;
	if (mLayer == "")
		return false;
	// implement me

	return true;
}
