/***************************************************************************
 *   Copyright (C) 2009 by Sergey Yakushev                                 *
 *   yakushevs@list.ru                                                     *
 *                                                                         *
 *   This is file define vrp plugins settings		                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef VRPPLUGIN_SETTINGS_H
#define VRPPLUGIN_SETTINGS_H

#include "units.h"

//  QT includes
#include <qstring.h>
#include <qlist.h>

// Qgis includes
#include "qgspoint.h"

// standart includes
#include <map>

// forward declaration Qgis-classes
class QgsVectorLayer;
class QgsFeature;

class SpeedUnit;
/**
@author Sergey Yakushev
*/
/**
 * \class VrpPluginSettings
 * \brief This class contained VRP plugin settings
 */
class VrpPluginSettings
{
public:
	/**
	 * \enum  DirectionType
	 * \brief DirectionType enumeration discribe 
	 */
	enum DirectionType {  FirstPointToLastPoint=1, LastPointToFirstPoint=2, Both=3 };
public:
	/**
	 * default constructor.
	 */
	VrpPluginSettings();
	
	/**
	 * destructor
	 */
	~VrpPluginSettings();
	/**
	 * This function test settings and return true if setting correct
	 */
	bool testSettings();
	/**
	 * return features direction type
	 */
	DirectionType directionType(QgsFeature& feature);
	/**
	 * return offroad speed in metric speed
	 */
	double offroadSpeed();
	/**
	 * return features speed in metric units
	 */
	double speed(QgsFeature& feature);
	/**
	 * return direction field id
	 */
	int directionFieldId();
	/**
	 * return speed field id
	 */
	int speedFieldId();
	/**
	 * return cost field id
	 */	
	QgsVectorLayer* roadsLayer();
	/**
	 * This function return default settings
	 */
	static VrpPluginSettings defaultSettings();
	/**
	 * set speed unit
	 */
	void setSpeedUnit( QString name );
	/**
	 * return speed unit
	 */
	SpeedUnit speedUnit() const;
public:
	/**
	 * contained Layer name
	 */
	QString mLayer;

	/**
	 * contained direction field name
	 */
	QString mDirection;

	/**
	 * contained Default direction
	 */
	DirectionType mDefaultDirection;

	/**
	 * contained speed filed name
	 */
	QString mSpeed;

	/**
	 * сontained default speed value;
	 */
	double mDefaultSpeed;
	/**
	 * contained off road speed value;
	 */
	double mOffroadSpeed;
	
private:
	/**
	 * cached direction field id
	 */
	int mDirectionFieldId;
	/**
	 * cached speed field id
	 */
	int mSpeedFieldId;
	/**
	 * cached cost field id
	 */
	int mCostFieldId;
	/**
	 * roads layer speed unit
	 */
	SpeedUnit mSpeedUnit;
};

#endif
