/***************************************************************************
 *   Copyright (C) 2009 by Sergey Yakushev                                 *
 *   yakushevs@list.ru                                                     *
 *                                                                         *
 *   This is a plugin generated from the QGIS plugin template              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

/**
 * \file directioneditormaptool.cpp
 * \brief implemetation direction editor maptool
 */


//qt includes
#include <qcombobox.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <qlabel.h>
#include <qlineedit.h>
#include <QToolButton>

// Qgis includes
#include <qgsmapcanvas.h>
#include <qgsmaptoolemitpoint.h>
#include <qgisinterface.h>
#include <qgsrubberband.h>
#include <qgsmaptopixel.h>

// VrpPlugin includes
#include "plugin.h"
#include "shortestpathwidget.h"
#include "utils.h"

//standard includes

VrpPluginShortestPathWidget::VrpPluginShortestPathWidget( QWidget* theParent, VrpPlugin *thePlugin ) :
	QDockWidget( theParent ), mPlugin( thePlugin )
{
	setWindowTitle( tr("Shortest path") );
	setObjectName( "ShortestPathDock" );
	setAllowedAreas( Qt::LeftDockWidgetArea | Qt::RightDockWidgetArea );

	QWidget *myWidget = new QWidget(this);
	setWidget( myWidget );

	QVBoxLayout *v = new QVBoxLayout( myWidget );
	QHBoxLayout *h = NULL;
	QLabel *l = NULL;
	
	l = new QLabel(tr("Start:"), myWidget );
	v->addWidget( l );
	h = new QHBoxLayout();
	mFrontPointLineEdit = new QLineEdit( myWidget );
	mFrontPointLineEdit->setReadOnly( true );
	QToolButton *selectFrontPoint = new QToolButton( myWidget );
	selectFrontPoint->setCheckable( true );
	selectFrontPoint->setIcon( QPixmap(":/vrp/coordinate_capture.png") );
	h->addWidget( mFrontPointLineEdit );
	h->addWidget( selectFrontPoint );
	v->addLayout(h);

	l = new QLabel(tr("Stop:"), myWidget );
	v->addWidget( l );
	h = new QHBoxLayout();
	mBackPointLineEdit = new QLineEdit( myWidget );
	mBackPointLineEdit->setReadOnly( true );
	QToolButton *selectBackPoint = new QToolButton( myWidget );
	selectBackPoint->setCheckable( true );
	selectBackPoint->setIcon( QPixmap(":/vrp/coordinate_capture.png") );	
	h->addWidget( mBackPointLineEdit );
	h->addWidget( selectBackPoint );
	v->addLayout( h );

	h = new QHBoxLayout( this );
	l = new QLabel(tr("Criterion:"), myWidget );
	mCriterionName = new QComboBox( myWidget );
	mCriterionName->insertItem(0, tr("Length"));
	mCriterionName->insertItem(1, tr("Time")  );
	h->addWidget( l );
	h->addWidget( mCriterionName );
	v->addLayout( h );
	
	h = new QHBoxLayout( myWidget );
	l = new QLabel( tr("Length:"), myWidget );
	mPathCostLineEdit = new QLineEdit( myWidget );
	mPathCostLineEdit->setReadOnly ( true );
	h->addWidget( l );
	h->addWidget( mPathCostLineEdit );
	v->addLayout( h );

	h = new QHBoxLayout( myWidget );
	l = new QLabel( tr("Time:"), myWidget );
	mPathTimeLineEdit = new QLineEdit( myWidget );
	mPathTimeLineEdit->setReadOnly ( true );
	h->addWidget( l );
	h->addWidget( mPathTimeLineEdit );
	v->addLayout( h );
	
	h = new QHBoxLayout( myWidget );
	mCalculate = new QPushButton( tr("Calculate"), myWidget );
	h->addWidget( mCalculate );
	mClear =  new QPushButton( tr("Clear"), myWidget );
	h->addWidget( mClear );
	v->addLayout( h );
	v->addStretch();

	mFrontPointMapTool = new QgsMapToolEmitPoint( mPlugin->iface()->mapCanvas() );
	mFrontPointMapTool->setButton ( selectFrontPoint );
	
	mBackPointMapTool  = new QgsMapToolEmitPoint( mPlugin->iface()->mapCanvas() );
	mBackPointMapTool->setButton ( selectBackPoint );

	connect( selectFrontPoint, SIGNAL( clicked(bool) ), this, SLOT( onSelectFrontPoint() ) );
	connect( mFrontPointMapTool, SIGNAL( canvasClicked( const QgsPoint&, Qt::MouseButton ) ), this, SLOT( setFrontPoint( const QgsPoint& ) ) );

	connect( selectBackPoint, SIGNAL( clicked(bool) ), this, SLOT( onSelectBackPoint() ) );
	connect( mBackPointMapTool, SIGNAL( canvasClicked( const QgsPoint&, Qt::MouseButton ) ), this, SLOT( setBackPoint( const QgsPoint& ) ) );

	connect( mCalculate, SIGNAL(clicked(bool) ), this, SLOT( findingPath() ) );
	connect( mClear, SIGNAL(clicked(bool)), this, SLOT(clear()) );
	
	mrbFrontPoint = new QgsRubberBand( mPlugin->iface()->mapCanvas(), true );
	mrbFrontPoint->setColor( Qt::green );
	mrbFrontPoint->setWidth( 2 );

	mrbBackPoint = new QgsRubberBand( mPlugin->iface()->mapCanvas(), true );
	mrbBackPoint->setColor( Qt::red );
	mrbBackPoint->setWidth( 2 );

	mrbPath = new QgsRubberBand( mPlugin->iface()->mapCanvas(), false );
	mrbPath->setWidth( 2 );
}
VrpPluginShortestPathWidget::~VrpPluginShortestPathWidget()
{
	delete mFrontPointMapTool;
	delete mBackPointMapTool;

	delete mrbFrontPoint;
	delete mrbBackPoint;
	delete mrbPath;
}
void VrpPluginShortestPathWidget::onSelectFrontPoint()
{
	mPlugin->iface()->mapCanvas()->setMapTool( mFrontPointMapTool );
}
void VrpPluginShortestPathWidget::setFrontPoint( const QgsPoint& pt )
{
	mPlugin->iface()->mapCanvas()->unsetMapTool( mFrontPointMapTool );
	mFrontPointLineEdit->setText( QString("(")+QString().setNum(pt.x()) + QString(",") + 
		QString().setNum(pt.y()) + QString(")") );
	mFrontPoint = pt;

	double mupp = mPlugin->iface()->mapCanvas()->getCoordinateTransform()->mapUnitsPerPixel()*2;

	mrbFrontPoint->reset(true);
	mrbFrontPoint->addPoint( QgsPoint( pt.x()-mupp, pt.y()-mupp), false );
	mrbFrontPoint->addPoint( QgsPoint( pt.x()+mupp, pt.y()-mupp), false );
	mrbFrontPoint->addPoint( QgsPoint( pt.x()+mupp, pt.y()+mupp), false );
	mrbFrontPoint->addPoint( QgsPoint( pt.x()-mupp, pt.y()+mupp), true );
	mrbFrontPoint->show();
}
void VrpPluginShortestPathWidget::onSelectBackPoint()
{
	mPlugin->iface()->mapCanvas()->setMapTool( mBackPointMapTool );
}
void VrpPluginShortestPathWidget::setBackPoint( const QgsPoint& pt )
{
	mPlugin->iface()->mapCanvas()->unsetMapTool( mBackPointMapTool );
	
	mBackPoint = pt;
	mBackPointLineEdit->setText( QString("(")+QString().setNum(pt.x()) + QString(",") + 
		QString().setNum(pt.y()) + QString(")") );
	
	double mupp = mPlugin->iface()->mapCanvas()->getCoordinateTransform()->mapUnitsPerPixel()*2;

	mrbBackPoint->reset(true);
	mrbBackPoint->addPoint( QgsPoint( pt.x()-mupp, pt.y()-mupp), false );
	mrbBackPoint->addPoint( QgsPoint( pt.x()+mupp, pt.y()-mupp), false );
	mrbBackPoint->addPoint( QgsPoint( pt.x()+mupp, pt.y()+mupp), false );
	mrbBackPoint->addPoint( QgsPoint( pt.x()-mupp, pt.y()+mupp), true );
	mrbBackPoint->show();

}
void VrpPluginShortestPathWidget::findingPath()
{
	if ( mFrontPointLineEdit->text().isNull() || mBackPointLineEdit->text().isNull() )
		return;
	QList< QgsPoint > l;
	l.push_back( mFrontPoint );
	l.push_back( mBackPoint );
	AdjacencyMatrix m = mPlugin->adjacencyMatrix(l);

	DijkstraFinder::OptimizationCriterion criterion = DijkstraFinder::byCost;
	if (mCriterionName->currentIndex() == 1)
		criterion = DijkstraFinder::byTime;
	
	
	DijkstraFinder f(m, DijkstraFinder::byCost);
	ArcPath p = f.find( mFrontPoint, mBackPoint );

	mPathCostLineEdit->setText(QString().setNum(p.cost()/mPlugin->settings()->speedUnit().distanceUnit().multipler()) 
		+ mPlugin->settings()->speedUnit().distanceUnit().name() );
	mPathTimeLineEdit->setText(QString().setNum(p.time()/mPlugin->settings()->speedUnit().timeUnit().multipler())
	+ mPlugin->settings()->speedUnit().timeUnit().name() );

	mrbPath->reset( false );
	QList< QgsPoint >::const_iterator it;
	for ( it=p.path().constBegin(); it != p.path().constEnd();)
	{
		QgsPoint pt = *it;
		++it;
		if ( it == p.path().constEnd() )
			mrbPath->addPoint( pt , true );
		else
			mrbPath->addPoint( pt , false );
	}
	
//	mPlugin->setPath( p );
}
void VrpPluginShortestPathWidget::clear()
{
//	mPlugin->clearPath();
	mFrontPointLineEdit->setText(QString());
	mrbFrontPoint->reset(true);
	mBackPointLineEdit->setText(QString());
	mrbBackPoint->reset(true);
	mrbPath->reset();
}
