/***************************************************************************
 *   Copyright (C) 2009 by Sergey Yakushev                                 *
 *   yakushevs@list.ru                                                     *
 *                                                                         *
 *   This is file define vrp plugins settings		                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef VRPPLUGIN_SHORTESTPATHDLG_H
#define VRPPLUGIN_SHORTESTPATHDLG_H

//  QT includes
#include <QDockWidget>

// Qgis includes
#include "qgspoint.h"

// standart includes

// forward declaration 

class QComboBox;
class QLineEdit;
class QPushButton;

class QgsRubberBand;
class QgsMapToolEmitPoint;
class QgsMapCanvas;

class VrpPlugin;
/**
@author Sergey Yakushev
*/
/**
 * \class VrpPluginShortestPathDlg
 * \brief This class implement user interface for finding shortest path between two points.
 */
class VrpPluginShortestPathWidget : public QDockWidget
{
	Q_OBJECT
public:
	/**
	 * Standart construct
	 */
	VrpPluginShortestPathWidget( QWidget * , VrpPlugin *);
	/**
	 * destructor
	 */
	~VrpPluginShortestPathWidget();
	
private slots:
	/**
	 * on canvas click mouse button
	 */
	void setFrontPoint( const QgsPoint& );
	/**
	 * on canvas click mouse button
	 */
	void setBackPoint( const QgsPoint& );
	/**
	 * Activate map tool for coordinate capture
	 */
	void onSelectFrontPoint();
	/**
	 * Activate map tool for coordinate capture
	 */
	void onSelectBackPoint();
	/**
	 * finding path
	 */
	void findingPath();
	/**
	 * clear
	 */
	void clear();	
private:
	/*
	 * I'm use line edit for show data. It can be copy-paste procedure:)
	 */

	/**
	 * This line edit show front points coordinates
	 */
	QLineEdit *mFrontPointLineEdit;
	/**
	 * This line edit show back points coordinates
	 */
	QLineEdit *mBackPointLineEdit;
	/**
	 * This combobox conteined criterion name
	 */
	QComboBox *mCriterionName;
	/**
	 * This line edit show length calculated path
	 */
	QLineEdit *mPathCostLineEdit;
	/**
	 * This line edit show time calculated path
	 */
	QLineEdit *mPathTimeLineEdit;

	/**
	 * this button called to find shortest path
	 */
	QPushButton *mCalculate;
	/**
	 * this button called to clear line edits and clar current path
	 */
	QPushButton *mClear;

	/**
	 * this map tool use for select coordinates
	 */
	QgsMapToolEmitPoint *mFrontPointMapTool;
	/**
	 * this map tool use for select coordinates
	 */
	QgsMapToolEmitPoint *mBackPointMapTool;

	/**
	 * pointer to Plugin
	 */
	VrpPlugin *mPlugin;
	/**
	 * Front point
	 */
	QgsPoint mFrontPoint;
	/**
	 * Back point
	 */
	QgsPoint mBackPoint;
	/**
	 * show front point
	 */
	QgsRubberBand *mrbFrontPoint;
	/**
	 * show back point
	 */
	QgsRubberBand *mrbBackPoint;
	/**
	 * show shortest path
	 */
	QgsRubberBand *mrbPath;
};
#endif
