/***************************************************************************
 *   Copyright (C) 2009 by Sergey Yakushev                                 *
 *   yakushevs@list.ru                                                     *
 *                                                                         *
 *   This is file implements Units classes			           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

/**
 * \file units.cpp
 * \brief implementation of VRP plugins utils
 */

#include "units.h"

//qt includes

// Qgis includes

//standard includes
#include <iostream>

Unit::Unit()
{
	mMultipler = 1;
}
Unit::Unit(const QString& name, double multipler) :
	mName(name), mMultipler(multipler)
{
}
QString Unit::name() const
{
	return mName;
}
double Unit::multipler() const
{
	return mMultipler;
}

SpeedUnit::SpeedUnit() :
	mTimeUnit("",1), mDistanceUnit("",1)
{
}
SpeedUnit::SpeedUnit( const Unit& distanceUnit, const Unit& timeUnit) :
	mTimeUnit( timeUnit ), mDistanceUnit( distanceUnit )
{
}

QString SpeedUnit::name() const
{ 
	if ( mDistanceUnit.name().isNull() || mTimeUnit.name().isNull() )
		return QString();
	return mDistanceUnit.name() + QString("/") + mTimeUnit.name(); 
}

double SpeedUnit::multipler() const
{ return mDistanceUnit.multipler()/mTimeUnit.multipler(); }

Unit SpeedUnit::timeUnit() const
{ return mTimeUnit; }

Unit SpeedUnit::distanceUnit() const
{  return mDistanceUnit; }


